import argparse
from datupie.function_maps import FUNCTION_MAP

def main():    
#if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument('command', choices=FUNCTION_MAP.keys())
    args = parser.parse_args()
    func = FUNCTION_MAP[args.command]
    func()


from datupie.core.templates import datupie_template
import json
import os
import re
import requests
import sys

class StartCommand:
    def __init__(self):
        self.uri = "http://35.172.203.237:8081/create"

    def startproject(self):
        new_project = input("Is this a new datup (data science) project [y/N]: ")
        if new_project == "y" or new_project == "yes" or new_project == "Y" :
            print("Choose the name of your project using lowercase names")
            print("this client will be append to your project name a <datup> prefix for IaC repository")
            print("this client will be append to your project name a <_datup> suffix for all services you need with the name of service you use")
            print("for example: <your-project-name><-datalake><-datup>")
            print("this is the simplest way to prevent errors with cloud providers services name")
            project_name = str(input("Which would be the name of project: ")).lower()
            if self.validated(project_name = project_name):
                data = self.parse_file(project_name)
                response = requests.post(self.uri, json=data)
                if response.text == "Succesfull created":
                    self.startproject()
                elif response.text == "This architecture is deployed, if you want update it invoke the right method":
                    print(response.text)
                else : print(response.text)
            else: 
                print("Something is wrong with the name you are choosing, please try again")
        elif new_project == "N" :
            print("Not configured yet")
            sys.exit(0)
        else : 
            print("Not valid")
            sys.exit(0)
    
    def validated(self, **kwards):        
        regex = re.compile("[@_!#$%^&*()<>?/\|}{~:]")
        if (regex.search(kwards["project_name"])) == None : return True
        else : return False

    def parse_file(self, project_name):        
        data = datupie_template.data()
        data["project"] = project_name
        data["file"] = project_name
        data["architecture"]["resource"]["aws_s3_bucket"]["project-datalake"]["bucket"] = f"{project_name}-datalake-datup"
        data["architecture"]["resource"]["aws_codecommit_repository"]["project-repository"]["repository_name"] = f"{project_name}-repository-datup"
        data["architecture"]["resource"]["aws_s3_bucket_object"]["dev_enviroment"]["bucket"] = f"{project_name}-datalake-datup"
        data["architecture"]["resource"]["aws_s3_bucket_object"]["prod_enviroment"]["bucket"] = f"{project_name}-datalake-datup"
        
        return data

     

        


from datupie.core.startproject import StartCommand
import requests
import os
import json

class DeployCommand(StartCommand):
    def __init__(self):
        StartCommand.__init__(self)
        self.uri = "http://35.172.203.237:8081/deploy"
        self.uri_status = "http://35.172.203.237:8081/status"
    
    def deployproject(self):
        name_project = input("Type the name of (data science) project you would to deploy: ")
        data = dict({'project':name_project, 'file':name_project})
        response = requests.post(self.uri, json=data)
    
        if response.text == "It seems that is already a architecture deployed":
            self.create_folder_projects(data, project_name = name_project)
        elif response.text == "Resources were not deployed":
            print("Some error ocurrs while deploying")
        else : print(response.text)

    def create_folder_projects(self,data, **kwards):
        create = str(input("Would you want to create a local project? [y/N]:"))
        uri = None
        if create == "y":
            uri = str(input("Please enter de uri where you want to save the project:"))        
            uri = uri.replace("\\","/")

            if os.path.exists(f"{uri}/{kwards['project_name']}"):
                print("The project exists in local")
            else:
                os.mkdir(f"{uri}/{kwards['project_name']}")
                main_dirs = [kwards['project_name'],"test"]
                secondary_dirs = ["clean","prepare","transform","inference","data"]
                for _dir in main_dirs:
                    os.mkdir(f"{uri}/{kwards['project_name']}/{_dir}")
                    with open(f"{uri}/{kwards['project_name']}/{_dir}/__init__.py","w") : pass
                    if _dir == kwards['project_name']:
                        with open(f"{uri}/{kwards['project_name']}/{_dir}/__main__.py","w") as main_file : 
                            main_file.write(f"from {kwards['project_name']}.clean import clean\n")
                            main_file.write(f"from {kwards['project_name']}.prepare import prepare\n")
                            main_file.write(f"from {kwards['project_name']}.transform import transform\n")
                            main_file.write(f"from {kwards['project_name']}.inference import inference\n\n")
                            main_file.write(f"if __name__ == '__main__':\n")
                            main_file.write(f"\t## your code here datupian")

                for _dir in secondary_dirs:
                    os.mkdir(f"{uri}/{kwards['project_name']}/{kwards['project_name']}/{_dir}")
                    if _dir == "data" : pass
                    else :
                        with open(f"{uri}/{kwards['project_name']}/{kwards['project_name']}/{_dir}/__init__.py","w") : pass
                        with open(f"{uri}/{kwards['project_name']}/{kwards['project_name']}/{_dir}/{_dir}.py","w") : pass
                        with open(f"{uri}/{kwards['project_name']}/{kwards['project_name']}/{_dir}/{_dir}.ipynb","w") as _file : 
                            json.dump({
                                "metadata": {
                                    "language_info": {
                                        "codemirror_mode": {
                                            "name": "ipython",
                                            "version": 3
                                        },
                                        "file_extension": ".py",
                                        "mimetype": "text/x-python",
                                        "name": "python",
                                        "nbconvert_exporter": "python",
                                        "pygments_lexer": "ipython3",
                                        "version": 3
                                    },
                                    "orig_nbformat": 2
                                },
                                "nbformat": 4,
                                "nbformat_minor": 2,
                                "cells": [{
                                    "cell_type": "code",
                                    "execution_count": "",
                                    "metadata": {},
                                    "outputs": [],
                                    "source": [
                                        "import datup as dt\n",
                                        "import pandas as pd\n",
                                        "import numpy as np"
                                    ]
                                }]
                            }, _file)

        response_status = requests.post(self.uri_status,json=data)
        if os.path.exists(f"{uri}/{kwards['project_name']}"):
            with open(f"{uri}/{kwards['project_name']}/{kwards['project_name']}.tf.json", "w") as _file:
                json.dump(response_status.json(),_file)
            with open(f"{uri}/{kwards['project_name']}/project.cfg", "w") as _file:
                _file.write("[metadata]\n")
                _file.write(f"project = {kwards['project_name']}\n")
                _file.write(f"architecture = {uri}/{kwards['project_name']}/{kwards['project_name']}.tf.json\n")                
        else : print(response_status.text)
  
    





import requests
import os
import json

class DestroyCommand:
    def __init__(self):
        self.uri = "http://35.172.203.237:8081/destroy"
    
    def destroyproject(self):
        name_project = input("Type the name of (data science) project you would to destroy: ")
        data = dict({'project':name_project, 'file':name_project})
        response = requests.post(self.uri, json=data)
        print(response.text)
        


def data():    
    return dict({
        "project": "Project",
        "file":"Project",
        "architecture": {
            "terraform": {
                "required_providers": {
                    "aws": {
                        "source": "hashicorp/aws"
                    }
                }
            },
            "provider": {
                "aws": {
                    "profile": "default",
                    "region": "us-east-1"
                }
            },
            "resource": {
                "aws_s3_bucket":{
                    "project-datalake":{
                        "bucket":"project-datalake",
                        "acl":"private"
                    }
                },
                "aws_codecommit_repository":{
                    "project-repository":{
                        "repository_name":"project-repository",
                        "description":"This is a repository created by DatupieHW under Terraform"
                    }
                },
                "aws_s3_bucket_object":{
                    "dev_enviroment":{
                        "bucket":"project-datalake",
                        "key":"dev/",
                        "force_destroy":True
                    },
                    "dev_enviroment_clean":{
                        "bucket":"project-datalake",
                        "key":"dev/clean/",
                        "force_destroy":True
                    },
                    "dev_enviroment_prepare":{
                        "bucket":"project-datalake",
                        "key":"dev/prepare/",
                        "force_destroy":True
                    },
                    "dev_enviroment_transform":{
                        "bucket":"project-datalake",
                        "key":"dev/transform/",
                        "force_destroy":True
                    },
                    "dev_enviroment_inference":{
                        "bucket":"project-datalake",
                        "key":"dev/inference/",
                        "force_destroy":True
                    },
                    "dev_enviroment_data":{
                        "bucket":"project-datalake",
                        "key":"dev/data/",
                        "force_destroy":True
                    },
                    "prod_enviroment":{
                        "bucket":"project-datalake",
                        "key":"prod/",
                        "force_destroy":True
                    },
                    "prod_enviroment_inference":{
                        "bucket":"project-datalake",
                        "key":"prod/inference/",
                        "force_destroy":True
                    }
                }
            }
        }
    })